;------------------------------------------------------------------------------
;	Programma per la lettura dei dati da una tastiera PS2
;
;	Il segnale di clock proveniente dalla tastiera va all'interrupt INT1. 
;	La routine legge 11 bit in formato seriale, verifica che il bit di start 
;	sia a zero, il bit di stop sia 1  e mette gli 8 bit di dato nei 
;   registri valid_code.
;	
;	Si utilizza un microcontrollore ATMEGA8 con freq. di clock = 8 MHz
;
;	Usa la porta D per comunicare con la tastiera
;
;	Collegamenti:
;	clock  -->  PD3 (INT1)
;	data   -->  PD4
;
;	Usa la porta C per azionare la memoria flash
;	
;	Collegamenti:
;	SCK  ->  PC0
;	SOUT ->  PC1
;	Sin  ->  PC2
;	CS   ->  PC3

;	Usa la porta B (pb0 - pb6) e la porta D (pd6 e pd7)
;	per inviare i dati all' AD converter

;   Ugo  Negro  2008-2009
;------------------------------------------------------------------------------
.include "m8def.inc"    

.def	temp		= r17
.def	cont		= r18
.def	contB		= r19
.def	bitcont		= r20
.def	dato		= r21
.def	scan_code	= r22
.def	valid_code	= r23
.def	old_code	= r24
.def	valid_old	= r25
.def	cont_pag	= r26
.def	npag		= r27
.def	pageaddrL	= r28
.def	pageaddrH	= r29
.def	flag_stato  = r16
.def	zero		= r4



.cseg

.org    0x0000
		rjmp	reset
							;Power-on and reset vector
.org	0x0002
		rjmp	readKB		;routine di gestione dell'interrupt 1

reset:  nop
		nop
		

		LDI   	R16,HIGH(RAMEND)   ;main program start
        OUT   	SPH,R16            ;set stack pointer to top of RAM
	  	LDI   	R16,LOW(RAMEND) 
	  	OUT   	SPL,R16 
		
        
		LDI   	R16,0b11111101
		OUT   	DDRC,R16  		; porta C  uscita tranne Sout della flash

		LDI   	R16,0xFF
		OUT   	DDRB,R16  		; porta B  uscita

		LDI   	R16,0b11000000
		OUT   	DDRD,R16  		; porta D bassa  ingresso
								; PD6 e PD7 sono uscite


		ldi		R16, 0x80		
		out		GICR, R16		; abilita INTR 1

		ldi		R16, 0x08		; INTR1 attiva sul fronte di discesa
		out		MCUCR, R16

		rcall	del30m

		ldi		bitcont, 11		; start + 8 dato + parit + stop
		ldi		dato, 0x00
		ldi		scan_code, 0x00
		ldi		old_code, 0x00

		clr     flag_stato
		clr		pageaddrL
		clr		pageaddrH
		sei						; abilita tutti gli INTERRUPT 
		
		ldi		npag,32
			
		rcall	del30m	


			
inizio:
		;call	del1m6
		rcall	del1m6
		rcall	del450n
		rcall	del450n
			
		tst     flag_stato		; verifica se  0
		breq    inizio
		
		
		cpi		valid_code, 0xE0
		breq	esteso

		rcall	tastoNM
		;out  portc, valid_code
		
		

		clr     flag_stato
		clr		valid_code
		rjmp	inizio

esteso:	

		clr     flag_stato
		clr		valid_code

		rcall	del1m6
		rcall	del1m6	
		
		

esteso1:
		tst     flag_stato		; verifica se  0
		breq	esteso1
		
		rcall	del1m6
		rcall	del1m6	
		
		
		out  	portB, valid_code
	    rcall    tastoEX
		
		
		
		
		clr     flag_stato
		clr		valid_code
		

		rjmp	inizio
	




; -------------------------------------------------------------------------
; Routine di gestione interrupt 1 per la lettura dei codici di tastiera


readKB:
		
		cpi 	bitcont, 11
		brne	sotto

		in		dato, pind
		andi 	dato, 0x10		; maschero i bit per leggere PD4
		cpi		dato, 0x00		; verifico che il bit di start sia zero

		breq 	fine			; se  1 esco dal ciclo
				
sotto:	

		in		dato, pind		
		andi 	dato, 0x10		; maschero i bit per leggere PD4
		cpi		bitcont,3		; se il contatore  maggiore di 2
		brge 	bit_valido 		; e minore di 11 metto il bit nel carry e
								; faccio scorrere il registro
		rjmp	sotto2

bit_valido:

		andi 	dato, 0x10		
		cpi		dato, 0x00
		breq	giu

		sec						; porta a uno il carry
		ror 	scan_code
		rjmp	fine
giu:	
		clc						; porta a zero il carry		
		ror 	scan_code
		rjmp	fine

sotto2:
		cpi		bitcont,2		; se il contatore  uguale a 2 (bit di parit)
		breq	fine
		
		cpi		bitcont,1		; se il contatore  uguale a 1 (bit di stop)
		breq	sotto3
		
		rjmp	fine
		
sotto3:
		mov		valid_old, valid_code
		mov		valid_code, scan_code
		ser		flag_stato		; porta a 1 flag di stato	
		ldi		bitcont, 11
		reti		 		

fine:	dec		bitcont
		
		reti



;----------------------------------------------------------------
;	Routine che accede alla tabella di look up per tasti con
;	codice normale

tastoNM:

	ldi		ZH, high(2*pagina)
	ldi		ZL, low(2*pagina)
	clr     zero
	clr		pageaddrL
	clr		pageaddrH

	add	 	ZL, valid_code
	adc     ZH, zero

	lpm
	
	mov		pageaddrL, R0		; il valore di indirizzo di pagina 
	lsl		pageaddrL			;  moltiplicato per 32 mediante 
	rol		pageaddrH			; 5 scorrimenti a sinistra
	lsl		pageaddrL
	rol		pageaddrH
	lsl		pageaddrL
	rol		pageaddrH
	lsl		pageaddrL
	rol		pageaddrH
	lsl		pageaddrL
	rol		pageaddrH
	
	rcall	leggi_mem
	sub		ZL,valid_code


	rcall  	del30m
	ldi		ZH, high(2*pagina)
	ldi		ZL, low(2*pagina)
	ret
;----------------------------------------------------------------
;	Routine che accede alla tabella di look up per tasti con
;	codice esteso

tastoEX:
	
	ldi		ZH, high(2*paginaEX)
	ldi		ZL, low(2*paginaEX)

	
	clr     zero
	clr		pageaddrL
	clr		pageaddrH

	add	 	ZL, valid_code
	adc     ZH, zero


	lpm
	
	mov		pageaddrL, R0		; il valore di indirizzo di pagina 
	lsl		pageaddrL			;  moltiplicato per 32 mediante 
	rol		pageaddrH			; 5 scorrimenti a sinistra
	lsl		pageaddrL
	rol		pageaddrH
	lsl		pageaddrL
	rol		pageaddrH
	lsl		pageaddrL
	rol		pageaddrH
	lsl		pageaddrL
	rol		pageaddrH

	rcall	leggi_mem
	sub		ZL,valid_code

	rcall  	del30m
	ldi		ZH, high(2*paginaEX)
	ldi		ZL, low(2*paginaEX)
	ret	



;-------------------------------------------------------------------
;	Routine che legge i campioni in memoria 
;	n pagine alla volta


Leggi_mem:
		nop
		rcall	del10m
		cbi		portC,3			; azzero CS
		nop


		nop
		ldi		dato, 0xE8		; OP code per lettura dalla
		rcall	wr_byte			; main memory 
		nop
		nop	

		mov		dato, pageaddrH	; ( 3 bit )
		rcall	wr_byte

		nop	
		mov		dato, pageaddrL	; (tutti 8 i bit)
		rcall	wr_byte

		nop
		nop
		ldi		dato, 0x00		; ( A7 - A0 )
		rcall	wr_byte
		nop
		nop
		ldi		dato, 0x00		; 
		rcall	wr_byte			; dummy bytes
		nop
		nop
		rcall	wr_byte			; dummy bytes
		nop
		nop
		rcall	wr_byte			; dummy bytes
		nop
		nop
		rcall	wr_byte			; dummy bytes
		nop
		nop
		nop
		
		mov		cont_pag, npag

iniz_ciclo:
		ldi		contB, 255		; ripeto il ciclo per 256 volte
		nop						; perch in 1 pagina ci sono 256 byte
		nop

ciclo_read:					
		nop					
		nop
		rcall	rd_byte
		out		PORTB, dato
		out		portD, dato

		rcall	del100u			; <----- ritardo per rispettare gli 
								; 8000 campioni al secondo
		cpi		contB, 0
		breq	ciclo_pag
		nop
		nop
		dec		contB
		rjmp	ciclo_read

ciclo_pag:
		
		cpi 	cont_pag,0
		breq	fine_ciclo
		dec		cont_pag
		rjmp	iniz_ciclo
		

fine_ciclo: 
		nop

		sbi		portC,3			; attivo CS

		clr     cont_pag
		clr     contB

		ret

;--------------------------------------------------------------------
;	WRITE  BYTE
;	Routine che converte un byte in forma seriale  
;	e lo invia alla memoria assieme al relativo clock                                                              
;   Il dato deve essere sincronizzato sul fronte si SALITA di ck

wr_byte:
		
		nop
		ldi		cont, 8		; contatore inizializzato a 8
		mov 	temp, dato
		nop

w_inizio:		
		cbi		portC,0		; azzero CK
		nop
		nop
		nop
		lsl		temp		; scorrimento a sinistra
		brcs	w_uno		; salta se il carry  1
		rjmp	w_zero
		

w_uno:	nop
		sbi		portC,2		; attivo SIN
		nop
		nop
		nop
		nop
		sbi		portC,0		; attivo CK
		nop
		nop
		rjmp	w_fine

w_zero:	nop
		cbi		portC,2		; azzero SIN
		nop
		nop
		nop
		nop
		sbi		portC,0		; attivo CK
		nop
		nop
		rjmp	w_fine

w_fine:	dec		cont
		cpi		cont,0
		brne	w_inizio
		nop
		nop
		
		ret


;----------------------------------------------------------------------
;	READ  BYTE
;	Routine che legge dalla memoria un byte in forma seriale
;	e lo converte in forma parallela. Il dato  sincronizzato con il
;	fronte di discesa del clock.

rd_byte:
		ldi		temp, 0x00
		ldi		dato, 0x00
		nop
		ldi		cont, 8	; contatore inizializzato a 8
		nop
			
		
r_inizio:
		
		
		nop
		nop
		nop
		nop
		nop
		nop	
		cbi		portC, 0	; azzero Ck
		nop
		nop
		nop
		nop
		nop
		nop
		nop

		in		dato, pinC

		andi	dato, 0b00000010	; maschero tutti i bit tranne bit1 (S_OUT)
		cpi 	dato, 0x00
		brne	r_uno
		rjmp	r_zero

r_uno:	nop
		sec					;porto a uno il carry
		rol		temp
		rjmp 	r_fine

r_zero: nop
		clc					;azzero il carry
		rol		temp
		rjmp	r_fine

r_fine:	
		nop
		sbi		portC, 0	; attivo il clock
		nop
		dec		cont
		cpi		cont,0
		brne	r_inizio
		mov		dato, temp
		nop
		nop

		ret


;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
;  Routine di ritardo
;  delay 30 ms

del30m:     push   r25        
            push   r26       
            push   r27

			ldi	   r25,15      
w15m_3:	    ldi    r26,78                             
w15m_2:	    ldi    r27,155                
w15m_1:     dec    r27       
			brne   w15m_1     
			dec    r26        
			brne   w15m_2     
			dec    r25
			brne   w15m_3

			pop    r27
			pop    r26         
			pop    r25        
            ret              

;---------------------------------------------------------------------------
; delay 10ms


del10m:     push   r18
            push   r19
			ldi    r18,125
w10m_2:		ldi    r19,255
w10m_1:		dec    r19
			nop
			brne   w10m_1 
            dec    r18
			brne   w10m_2
			pop    r19
			pop    r18
			ret
;---------------------------------------------------------------------------
; delay 100us

del100u:    push   r18
            push   r19
			ldi    r18,2
w100u_2:	ldi    r19,127
w100u_1:	dec    r19
			brne   w100u_1 
            dec    r18
			brne   w100u_2
			pop    r19
			pop    r18
			ret


; delay 450ns
del450n:    push   r18
            ldi    r18,4
w450n:	    dec    r18
			brne   w450n
            pop    r18
			ret


; delay 1.6ms
del1m6:     push   r18        
            push   r19        
            ldi    r18,33     
w1m6_2:		ldi    r19,255    
w1m6_1:	    dec    r19               
			brne   w1m6_1     
			dec    r18        
			brne   w1m6_2     
			pop    r19        
			pop    r18       
            ret     


;-------------------------------------------------------------------------             
; La tabella di look up contiene l'indirizzo della pagina diviso per 32
;		0 1 2 3			  4 5 6 7    		8 9 A B 		C D E F
pagina:

.db		60,60,60,60,      60,60,60,60,      60,60,60,60,    60,53,60,60
.db		1,45,37,6,        44,16,27,1,       1,64,25,18,     0,22,28,1
.db		2,2,23,3,         4,30,29,2,        2,38,21,5,      19,17,31,2
.db		3,13,1,7,         6,24,32,3,        3,3,12,9,       20,33,34,3
.db		4,39,10,8,        14,26,35,4,       4,40,41,11,     4,15,4,4
.db		5,5,5,5,          5,5,5,5,          5,37,36,42,     5,5,5,5
.db		6,6,6,6,          6,6,6,6,          6,27,6,30,      33,6,6,6
.db		26,40,28,31,      32,34,7,7,        7,42,29,41,     43,35,7,7



;-------------------------------------------------------------------------             
; La tabella di look up contiene l'indirizzo della pagina diviso per 32
;		0 1 2 3			  4 5 6 7    		8 9 A B 		C D E F
paginaEX:

.db		0,1,0,0,          0,39,0,0,         0,0,0,0,        0,0,0,0
.db		1,45,37,6,        44,16,27,1,       1,64,25,18,     0,22,28,1
.db		2,2,23,3,         4,30,29,2,        2,38,21,5,      19,17,31,2
.db		3,13,1,7,         6,24,32,3,        3,3,12,9,       20,33,34,3
.db		4,39,10,8,        14,26,35,4,       4,40,41,11,     4,15,4,4
.db		5,5,5,5,          5,5,5,5,          5,37,36,42,     5,5,5,5
.db		6,6,6,6,          6,6,6,6,          6,27,6,50,      33,6,6,6
.db		46,47,52,31,      49,51,7,7,        51,42,29,41,    43,35,7,7
